#!/usr/bin/ruby1.9
# -*- coding: utf-8 -*-

require File.dirname(__FILE__) + '/dissect.rb'

dissect "Las Vegas"
dissect "99¢"
dissect "好"

dissect "Enjoy 鋸".encode("UTF-16LE")
dissect "a\x00b\x00c\x00".force_encoding("UTF-16LE")
dissect "a\x00b\x00\xA2\x00".encode("UTF-8", "UTF-16LE")

dissect "ab\xC2\xFFfg¢"
dissect "ab\xC2\xFFfg¢".encode("UTF-8", "UTF-8-MAC", :undef=>:replace, :invalid=>:replace, :replace=>"\uFFFD")

dissect "好 costs 99¢\xFE".encode("US-ASCII", :xml=>:text, :invalid=>:replace)

############## SOCKETS ############################################
# Strings are sent as they are (bytes aren't changed)
# Received strings are marked as ASCII-8BIT.
dissect send_through_socket("A 99¢ 好")

############## BUG WITH CONCAT ####################################
# http://stackoverflow.com/questions/4236793/why-are-two-strings-with-same-bytes-and-encoding-not-identical-in-ruby-1-9

############## EXTERNAL/INTERNAL ENCODING ########################
# http://blog.grayproductions.net/articles/ruby_19s_three_default_encodings

open("data.txt", "w:UTF-16LE:UTF-8") do |file|
  file << "57¢"
end

open("data.txt", "r:UTF-16LE:UTF-8") do |file|
  p file.external_encoding
  p file.internal_encoding
  file.each { |line| p [line.encoding.name, line] }
end
dissect File.read("data.txt")

############## DEFAULT EXTERNAL/INTERNAL ENCODING #################
#
# Default external/internal encoding: ruby -E ASCII:ASCII
#  - affects the output of .inspect
#  - doesn't seem to affect $stdout

p Encoding.default_internal
p Encoding.default_external
p $stdout.external_encoding

Encoding.default_external = "UTF-8"
Encoding.default_internal = "UTF-8"
Encoding.default_internal = nil

############## COMPATIBLE ENCODINGS ###############################
# From the source code of ruby, encoding.c:
# Encoding.compatible?
#   Checks the compatibility of two strings.
#   If they are compatible, means concatenatable,
#   returns an encoding which the concatenated string will be.
#   If they are not compatible, nil is returned.
p "COMPATIBILITY"
p Encoding.compatible?("¢", "a")  # UTF-8
p Encoding.compatible?("a", "¢")  # UTF-8
p Encoding.compatible?("a", "a")  # UTF-8

p Encoding.compatible?("a".force_encoding("US-ASCII"), "a") # US-ASCII
p Encoding.compatible?("a", "a".force_encoding("US-ASCII")) # UTF-8

p Encoding.compatible?("¢", "a".force_encoding("US-ASCII")) # UTF-8
p Encoding.compatible?("a".force_encoding("US-ASCII"), "¢") # UTF-8

p Encoding.compatible?("a", "a\x00".force_encoding("UTF-16LE")) # nil

############### ADDING STRINGS ####################################
# Ruby tries to use the encoding of the left-hand string if it can.
p ("a".force_encoding("US-ASCII")+"a").encoding   # US-ASCII
p ("a"+"a".force_encoding("US-ASCII")).encoding   # UTF-8

############## COMPARING STRINGS ##################################
# rb_str_equal:  Returns false if they aren't comparable.
#  Otherwise returns true if bytes are equal.
#
# rb_str_comparable:
#  If either is empty, return true.
#  If same encoding, return true.
#  If they both have 7-bit code range, return true.
#  If one has a 7-bit code range and the other is ASCII compatible,
#  return true.
p "abcd"=="abcd".force_encoding("UTF-16LE")
p "abcd"=="abcd".force_encoding("US-ASCII")
p "a\x00"=="a\x00".force_encoding("UTF-16LE")

############## LIST OF ENCODINGS ##################################
p Encoding.name_list

############## PITFALLS ###########################################
# Strings can be inconsistent - can't get ordinals of characters, can't transcode

# Comparisons are done on a byte-by-byte basis

# Complex rules for string operations
# http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-core/20540


############## CONTRAST WITH PYTHON ###############################
# string:    immutable list of Unicode code units  ( 'abc' )
# bytes:     immutable list of bytes ( b'abc' )
# bytearray: mutable list of bytes
#
#              Socket recv returns bytes object.

